require 'spec_helper'

describe 'Entry' do
  before(:context) do
    @proposer = UserHelper.make(8)
    @proposed = UserHelper.make(9)
    ResourceHelper.safe_delete(@proposer)
    ResourceHelper.safe_delete(@proposed)
    @proposer.create
    @proposed.create
    @partnership = CyberCoach::Partnership.new
    @partnership.proposer = @proposer
    @partnership.proposed = @proposed
    @partnership.privacy_level = CyberCoach::PrivacyLevel::REGISTERED_USER
    ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    ResourceHelper.safe_delete(@partnership, @proposed.authentication)
    @partnership.create(@proposer.authentication)
    @partnership.update(@proposed.authentication)
    @sport = CyberCoach::Sport.new
    @sport.name = 'Running'
    @sport.read
    @subscription = CyberCoach::Subscription.new
    @subscription.subscriber = @partnership
    @subscription.sport = @sport
    @subscription.privacy_level = CyberCoach::PrivacyLevel::REGISTERED_USER
    @subscription.create(@proposer.authentication)
  end

  after(:context) do
    ResourceHelper.safe_delete(@subscription, @proposer.authentication)
    ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    ResourceHelper.safe_delete(@partnership, @proposed.authentication)
    ResourceHelper.safe_delete(@proposer)
    ResourceHelper.safe_delete(@proposed)
  end

  before(:example) do
    @entry = CyberCoach::Entry.new
    @entry.subscription = @subscription
    @entry.comment = 'I like pants'
    @entry.location = 'In your pants'
    @entry.duration = 1
    @entry.privacy_level = CyberCoach::PrivacyLevel::REGISTERED_USER
  end

  it 'should instantiate itself with preset attributes' do
    uri = '/pants/'
    id = 1
    subscription = CyberCoach::Subscription.new
    comment = 'those pants are fancy'
    location = 'in your pants'
    duration = 123
    privacy_level = CyberCoach::PrivacyLevel::OWNER
    pants = 'yes'
    @entry = CyberCoach::Entry.new(
      uri: uri,
      id: id,
      subscription: subscription,
      comment: comment,
      location: location,
      duration: duration,
      privacy_level: privacy_level,
      pants: pants
    )
    expect(@entry.uri).to be(uri)
    expect(@entry.id).to be(id)
    expect(@entry.subscription).to be(subscription)
    expect(@entry.comment).to be(comment)
    expect(@entry.location).to be(location)
    expect(@entry.duration).to be(duration)
    expect(@entry.privacy_level).to be(privacy_level)
    expect { @entry.pants }.to raise_error(NoMethodError)
  end

  it 'should raise an error if it does not exist' do
    @entry.id = 1
    expect { @entry.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
  end

  describe 'crud' do
    before(:example) do
      ResourceHelper.safe_delete(@entry, @proposer.authentication)
    end

    after(:example) do
      ResourceHelper.safe_delete(@entry, @proposer.authentication)
    end

    it 'should create itself' do
      @entry.create(@proposer.authentication)
      expect(@entry.id).not_to be_nil
    end

    it 'should read itself' do
      @entry.create(@proposer.authentication)
      @entry.read(@proposer.authentication)
      expect(@entry.id).not_to be_nil
      expect(@entry.uri).not_to be_nil
      expect(@entry.subscription.uri).to eq(@subscription.uri)
      expect(@entry.comment).not_to be_nil
      expect(@entry.location).not_to be_nil
      expect(@entry.duration).not_to be_nil
      expect(@entry.privacy_level).not_to be_nil
      expect(@entry.date_created).not_to be_nil
      expect(@entry.date_modified).not_to be_nil
    end

    it 'should update itself' do
      @entry.create(@proposer.authentication)
      old_privacy_level = @entry.privacy_level
      new_privacy_level = CyberCoach::PrivacyLevel::EVERYBODY
      @entry.privacy_level = new_privacy_level
      @entry.update(@proposer.authentication)
      expect(@entry.privacy_level).not_to eq(old_privacy_level)
      expect(@entry.privacy_level).to eq(new_privacy_level)
    end

    it 'should delete itself' do
      @entry.create(@proposer.authentication)
      @entry.delete(@proposer.authentication)
      expect { @entry.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
    end
  end

  describe 'invalidation' do
    before(:example) do
      ResourceHelper.safe_delete(@entry, @proposer.authentication)
    end

    after(:example) do
      ResourceHelper.safe_delete(@entry, @proposer.authentication)
    end

    xit 'should not invalidate when creating' do
      # TODO crap, this impossible to test...
      id = @entry.id
      @entry.id = 1234
      expect(@entry.id).not_to eq(id)
      expect { @entry.create(@proposer.authentication, false) }.not_to raise_error
      @entry.id = id
    end

    it 'should not invalidate when reading' do
      @entry.create(@proposer.authentication)
      id = @entry.id
      @entry.id = 1234
      expect(@entry.id).not_to eq(id)
      expect { @entry.read(@proposer.authentication, false) }.not_to raise_error
      @entry.id = id
    end

    it 'should not invalidate when updating' do
      @entry.create(@proposer.authentication)
      id = @entry.id
      @entry.id = 1234
      expect(@entry.id).not_to eq(id)
      expect { @entry.update(@proposer.authentication, false) }.not_to raise_error
      @entry.id = id
    end

    it 'should not invalidate when deleting' do
      @entry.create(@proposer.authentication)
      id = @entry.id
      @entry.id = 1234
      expect(@entry.id).not_to eq(id)
      expect { @entry.delete(@proposer.authentication, false) }.not_to raise_error
      @entry.id = id
    end
  end
end
