require 'spec_helper'

describe 'Partnership' do
  before(:context) do
    @proposer = UserHelper.make(8)
    @proposed = UserHelper.make(9)
    ResourceHelper.safe_delete(@proposer)
    ResourceHelper.safe_delete(@proposed)
    @proposer.create
    @proposed.create
  end

  after(:context) do
    ResourceHelper.safe_delete(@proposer)
    ResourceHelper.safe_delete(@proposed)
  end

  before(:example) do
    @partnership = CyberCoach::Partnership.new
    @partnership.proposer = @proposer
    @partnership.proposed = @proposed
    @partnership.privacy_level = CyberCoach::PrivacyLevel::REGISTERED_USER
  end

  it 'should instantiate itself with preset attributes' do
    uri = '/pants/'
    id = 1
    proposer = CyberCoach::User.new
    proposed = CyberCoach::User.new
    privacy_level = CyberCoach::PrivacyLevel::OWNER
    pants = 'yes'
    @partnership = CyberCoach::Partnership.new(
      uri: uri,
      id: id,
      proposer: proposer,
      proposed: proposed,
      privacy_level: privacy_level,
      pants: pants
    )
    expect(@partnership.uri).to be(uri)
    expect(@partnership.id).to be(id)
    expect(@partnership.proposer).to be(proposer)
    expect(@partnership.proposed).to be(proposed)
    expect(@partnership.privacy_level).to be(privacy_level)
    expect { @partnership.pants }.to raise_error(NoMethodError)
  end

  it 'should raise an error if it does not exist' do
    expect { @partnership.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
  end

  it 'should read all' do
    partnerships = CyberCoach::Partnership.read_all
    expect(partnerships.resources).not_to be_empty
    expect(partnerships.next).not_to be_nil
    expect { partnerships.previous }.to raise_error(CyberCoach::ResourcePage::NoPreviousPageError)
  end

  describe 'crud' do
    before(:example) do
      ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    end

    after(:example) do
      ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    end

    it 'should create itself' do
      @partnership.create(@proposer.authentication)
      expect(@partnership.id).not_to be_nil
    end

    it 'should read itself' do
      @partnership.create(@proposer.authentication)
      @partnership.read(@proposer.authentication)
      expect(@partnership.id).not_to be_nil
      expect(@partnership.uri).not_to be_nil
      expect(@partnership.proposer.uri).to eq(@proposer.uri)
      expect(@partnership.proposed.uri).to eq(@proposed.uri)
      expect(@partnership.confirmed_by_proposer).to be_truthy
      expect(@partnership.confirmed_by_proposed).to be_falsy
      expect(@partnership.privacy_level).not_to be_nil
      expect(@partnership.subscriptions).to be_empty
    end

    it 'should update itself' do
      @partnership.create(@proposer.authentication)
      old_privacy_level = @partnership.privacy_level
      new_privacy_level = CyberCoach::PrivacyLevel::EVERYBODY
      @partnership.privacy_level = new_privacy_level
      @partnership.update(@proposer.authentication)
      expect(@partnership.privacy_level).not_to eq(old_privacy_level)
      expect(@partnership.privacy_level).to eq(new_privacy_level)
    end

    it 'should delete itself' do
      @partnership.create(@proposer.authentication)
      @partnership.delete(@proposer.authentication)
      expect { @partnership.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
    end
  end

  describe 'invalidation' do
    before(:example) do
      ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    end

    after(:example) do
      ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    end

    it 'should not invalidate when creating' do
      proposer = @partnership.proposer
      @partnership.proposer = CyberCoach::User.new
      expect(@partnership.proposer).not_to eq(proposer)
      expect { @partnership.create(@proposer.authentication, false) }.not_to raise_error
      @partnership.proposer = proposer
    end

    it 'should not invalidate when reading' do
      @partnership.create(@proposer.authentication)
      proposer = @partnership.proposer
      @partnership.proposer = CyberCoach::User.new
      expect(@partnership.proposer).not_to eq(proposer)
      expect { @partnership.read(@proposer.authentication, false) }.not_to raise_error
      @partnership.proposer = proposer
    end

    it 'should not invalidate when updating' do
      @partnership.create(@proposer.authentication)
      proposer = @partnership.proposer
      @partnership.proposer = CyberCoach::User.new
      expect(@partnership.proposer).not_to eq(proposer)
      expect { @partnership.update(@proposer.authentication, false) }.not_to raise_error
      @partnership.proposer = proposer
    end

    it 'should not invalidate when deleting' do
      @partnership.create(@proposer.authentication)
      proposer = @partnership.proposer
      @partnership.proposer = CyberCoach::User.new
      expect(@partnership.proposer).not_to eq(proposer)
      expect { @partnership.delete(@proposer.authentication, false) }.not_to raise_error
      @partnership.proposer = proposer
    end
  end
end
