require 'spec_helper'

describe 'Subscription' do
  before(:context) do
    @proposer = UserHelper.make(8)
    @proposed = UserHelper.make(9)
    ResourceHelper.safe_delete(@proposer)
    ResourceHelper.safe_delete(@proposed)
    @proposer.create
    @proposed.create
    @partnership = CyberCoach::Partnership.new
    @partnership.proposer = @proposer
    @partnership.proposed = @proposed
    @partnership.privacy_level = CyberCoach::PrivacyLevel::REGISTERED_USER
    ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    ResourceHelper.safe_delete(@partnership, @proposed.authentication)
    @partnership.create(@proposer.authentication)
    @partnership.update(@proposed.authentication)
    @sport = CyberCoach::Sport.new
    @sport.name = 'Running'
    @sport.read
  end

  after(:context) do
    ResourceHelper.safe_delete(@partnership, @proposer.authentication)
    ResourceHelper.safe_delete(@partnership, @proposed.authentication)
    ResourceHelper.safe_delete(@proposer)
    ResourceHelper.safe_delete(@proposed)
  end

  before(:example) do
    @subscription = CyberCoach::Subscription.new
    @subscription.subscriber = @partnership
    @subscription.sport = @sport
    @subscription.privacy_level = CyberCoach::PrivacyLevel::REGISTERED_USER
  end

  it 'should instantiate itself with preset attributes' do
    uri = '/pants/'
    id = 1
    subscriber = CyberCoach::User.new
    sport = CyberCoach::Sport.new
    privacy_level = CyberCoach::PrivacyLevel::OWNER
    pants = 'yes'
    @subscription = CyberCoach::Subscription.new(
      uri: uri,
      id: id,
      subscriber: subscriber,
      sport: sport,
      privacy_level: privacy_level,
      pants: pants
    )
    expect(@subscription.uri).to be(uri)
    expect(@subscription.id).to be(id)
    expect(@subscription.subscriber).to be(subscriber)
    expect(@subscription.sport).to be(sport)
    expect(@subscription.privacy_level).to be(privacy_level)
    expect { @subscription.pants }.to raise_error(NoMethodError)
  end

  it 'should raise an error if it does not exist' do
    expect { @subscription.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
  end

  describe 'crud' do
    before(:example) do
      ResourceHelper.safe_delete(@subscription, @proposer.authentication)
    end

    after(:example) do
      ResourceHelper.safe_delete(@subscription, @proposer.authentication)
    end

    describe 'with partnership subscriber' do
      before(:example) do
        @subscription.subscriber = @partnership
      end

      it 'should create itself' do
        @subscription.create(@proposer.authentication)
        expect(@subscription.id).not_to be_nil
      end

      it 'should read itself' do
        @subscription.create(@proposer.authentication)
        @subscription.read(@proposer.authentication)
        expect(@subscription.id).not_to be_nil
        expect(@subscription.uri).not_to be_nil
        expect(@subscription.subscriber.uri).to eq(@partnership.uri)
        expect(@subscription.sport.uri).to eq(@sport.uri)
        expect(@subscription.entries).to be_empty
        expect(@subscription.privacy_level).not_to be_nil
        expect(@subscription.date_created).not_to be_nil
      end

      it 'should update itself' do
        @subscription.create(@proposer.authentication)
        old_privacy_level = @subscription.privacy_level
        new_privacy_level = CyberCoach::PrivacyLevel::EVERYBODY
        @subscription.privacy_level = new_privacy_level
        @subscription.update(@proposer.authentication)
        expect(@subscription.privacy_level).not_to eq(old_privacy_level)
        expect(@subscription.privacy_level).to eq(new_privacy_level)
      end

      it 'should delete itself' do
        @subscription.create(@proposer.authentication)
        @subscription.delete(@proposer.authentication)
        expect { @subscription.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
      end
    end

    describe 'with user subscriber' do
      before(:example) do
        @subscription.subscriber = @proposer
      end

      it 'should create itself' do
        @subscription.create(@proposer.authentication)
        expect(@subscription.id).not_to be_nil
      end

      it 'should read itself' do
        @subscription.create(@proposer.authentication)
        @subscription.read(@proposer.authentication)
        expect(@subscription.id).not_to be_nil
        expect(@subscription.uri).not_to be_nil
        expect(@subscription.subscriber.uri).to eq(@proposer.uri)
        expect(@subscription.sport.uri).to eq(@sport.uri)
        expect(@subscription.entries).to be_empty
        expect(@subscription.privacy_level).not_to be_nil
        expect(@subscription.date_created).not_to be_nil
      end

      it 'should update itself' do
        @subscription.create(@proposer.authentication)
        old_privacy_level = @subscription.privacy_level
        new_privacy_level = CyberCoach::PrivacyLevel::EVERYBODY
        @subscription.privacy_level = new_privacy_level
        @subscription.update(@proposer.authentication)
        expect(@subscription.privacy_level).not_to eq(old_privacy_level)
        expect(@subscription.privacy_level).to eq(new_privacy_level)
      end

      it 'should delete itself' do
        @subscription.create(@proposer.authentication)
        @subscription.delete(@proposer.authentication)
        expect { @subscription.read(@proposer.authentication) }.to raise_error(CyberCoach::HttpError)
      end
    end
  end

  describe 'invalidation' do
    before(:example) do
      @subscription.subscriber = @proposer
      ResourceHelper.safe_delete(@subscription, @proposer.authentication)
    end

    after(:example) do
      ResourceHelper.safe_delete(@subscription, @proposer.authentication)
    end

    it 'should not invalidate when creating' do
      subscriber = @subscription.subscriber
      @subscription.subscriber = CyberCoach::User.new
      expect(@subscription.subscriber).not_to eq(subscriber)
      expect { @subscription.create(@proposer.authentication, false) }.not_to raise_error
      @subscription.subscriber = subscriber
    end

    it 'should not invalidate when reading' do
      @subscription.create(@proposer.authentication)
      subscriber = @subscription.subscriber
      @subscription.subscriber = CyberCoach::User.new
      expect(@subscription.subscriber).not_to eq(subscriber)
      expect { @subscription.read(@proposer.authentication, false) }.not_to raise_error
      @subscription.subscriber = subscriber
    end

    it 'should not invalidate when updating' do
      @subscription.create(@proposer.authentication)
      subscriber = @subscription.subscriber
      @subscription.subscriber = CyberCoach::User.new
      expect(@subscription.subscriber).not_to eq(subscriber)
      expect { @subscription.update(@proposer.authentication, false) }.not_to raise_error
      @subscription.subscriber = subscriber
    end

    it 'should not invalidate when deleting' do
      @subscription.create(@proposer.authentication)
      subscriber = @subscription.subscriber
      @subscription.subscriber = CyberCoach::User.new
      expect(@subscription.subscriber).not_to eq(subscriber)
      expect { @subscription.delete(@proposer.authentication, false) }.not_to raise_error
      @subscription.subscriber = subscriber
    end
  end
end
