require 'spec_helper'

describe 'User' do
  before(:example) do
    @user = CyberCoach::User.new
  end

  it 'should instantiate itself with preset attributes' do
    uri = '/pants/'
    id = 1
    username = 'pants'
    email = 'fancy@pants.com'
    password = 'fancy'
    real_name = 'Fancy Pants'
    privacy_level = CyberCoach::PrivacyLevel::OWNER
    pants = 'yes'
    @user = CyberCoach::User.new(
      uri: uri,
      id: id,
      username: username,
      email: email,
      password: password,
      real_name: real_name,
      privacy_level: privacy_level,
      pants: pants
    )
    expect(@user.uri).to be(uri)
    expect(@user.id).to be(id)
    expect(@user.username).to be(username)
    expect(@user.email).to be(email)
    expect(@user.password).to be(password)
    expect(@user.real_name).to be(real_name)
    expect(@user.privacy_level).to be(privacy_level)
    expect { @user.pants }.to raise_error(NoMethodError)
  end

  it 'should raise an error if it does not exist' do
    @user.username = 'gnirloff'
    expect { @user.read }.to raise_error(CyberCoach::HttpError)
  end

  it 'should read all' do
    users = CyberCoach::User.read_all
    expect(users.resources.size).to eq(5)
    expect(users.next).not_to be_nil
    expect { users.previous }.to raise_error(CyberCoach::ResourcePage::NoPreviousPageError)
  end

  it 'should get the authentication options' do
    @user = UserHelper.make(8)
    @user.create
    expect(@user.authentication).to eq(
                                      basic_auth: {
                                        username: @user.username,
                                        password: @user.password
                                      }
                                    )
  end

  describe 'crud' do
    before(:example) do
      @user = UserHelper.make(8)
      ResourceHelper.safe_delete(@user)
    end

    after(:example) do
      ResourceHelper.safe_delete(@user)
    end

    it 'should create itself without nilling the password' do
      old_uri = @user.uri
      old_password = @user.password
      @user.create
      expect(@user.uri).to eq(old_uri)
      expect(@user.password).to eq(old_password)
    end

    it 'should read itself without nilling the password' do
      old_password = @user.password
      @user.create
      @user.read
      # users do not have an id
      expect(@user.id).to be_nil
      expect(@user.uri).not_to be_nil
      expect(@user.username).not_to be_nil
      expect(@user.email).not_to be_nil
      expect(@user.password).to eq(old_password)
      expect(@user.privacy_level).not_to be_nil
      expect(@user.real_name).not_to be_nil
      expect(@user.date_created).not_to be_nil
    end

    it 'should update itself without nilling the password' do
      @user.create
      old_password = @user.password
      old_real_name = @user.real_name
      new_real_name = 'habakuck'
      @user.real_name = new_real_name
      @user.update
      expect(@user.password).to eq(old_password)
      expect(@user.real_name).not_to eq(old_real_name)
      expect(@user.real_name).to eq(new_real_name)
    end

    it 'should delete itself' do
      @user.create
      @user.delete
      expect { @user.read }.to raise_error(CyberCoach::HttpError)
    end
  end

  describe 'invalidation' do
    before(:example) do
      @user = UserHelper.make(8)
      ResourceHelper.safe_delete(@user)
    end

    after(:example) do
      ResourceHelper.safe_delete(@user)
    end

    it 'should not invalidate when creating' do
      password = @user.password
      @user.password = 'notThePassword'
      expect(@user.password).not_to eq(password)
      expect { @user.create({}, false) }.not_to raise_error
      @user.password = password
    end

    it 'should not invalidate when reading' do
      @user.create
      password = @user.password
      @user.password = 'notThePassword'
      expect(@user.password).not_to eq(password)
      expect { @user.read({}, false) }.not_to raise_error
      @user.password = password
    end

    it 'should not invalidate when updating' do
      @user.create
      password = @user.password
      @user.password = 'notThePassword'
      expect(@user.password).not_to eq(password)
      expect { @user.update({}, false) }.not_to raise_error
      @user.password = password
    end

    it 'should not invalidate when deleting' do
      @user.create
      password = @user.password
      @user.password = 'notThePassword'
      expect(@user.password).not_to eq(password)
      expect { @user.delete({}, false) }.not_to raise_error
      @user.password = password
    end
  end
end
